#!/usr/bin/env python

import QtConfigurator.Forms
import XSLForms.Prepare
import factory
import qtui
import qtxmldom
import os

class Configurator(QtConfigurator.Forms.Configurator):

    # Standard attributes.

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")

    template_resources = {
        "configuration" : "config.ui"
        }

    widget_resources = {
        "hard-disks" : ("config_hard_disks.ui", "hard_disks"),
        "memory-units" : ("config_memory_units.ui", "memory_units"),
        "storage-units" : ("config_storage_units.ui", "storage_units")
        }

    document_resources = {
        "base-system" : "config_base_system.xml",
        "cpu" : "config_cpu.xml",
        "hard-disk" : "config_hard_disk.xml",
        "keyboard" : "config_keyboard.xml",
        "mouse" : "config_mouse.xml",
        "screen" : "config_screen.xml"
        }

    # Helper methods.
    # NOTE: Should be moved to a superclass.

    def prepare_document(self, document_identifier):
        filename = self.document_resources[document_identifier]
        return os.path.abspath(os.path.join(self.resource_dir, filename))

    def get_document(self, document_identifier):
        return qtxmldom.parse(self.prepare_document(document_identifier))

    def get_elements(self, document_identifier):
        doc = self.get_document(document_identifier)
        return doc.getElementsByTagName(document_identifier + "-enum")

    def populate_list(self, field, elements):
        current_text = field.currentText()
        while field.count() > 0:
            field.removeItem(0)
        item = 0
        set = 0
        for element in elements:
            text = element.getAttribute("value")
            field.insertItem(text)
            if text == current_text:
                field.setCurrentItem(item)
                set = 1
            item += 1
        if not set:
            field.setCurrentItem(0)

    def prepare_template(self, template_identifier):
        filename = self.template_resources[template_identifier]
        return os.path.abspath(os.path.join(self.resource_dir, filename))

    def prepare_widget(self, template_identifier, widget_identifier):
        template_path = self.prepare_template(template_identifier)
        fragment_name, widget_name = self.widget_resources[widget_identifier]
        fragment_path = os.path.abspath(os.path.join(self.resource_dir, fragment_name))
        XSLForms.Prepare.ensure_qt_fragment(template_path, fragment_path, widget_name)
        return qtui.QWidgetFactory.create(fragment_path)

    # Initialisation.

    def __init__(self, *args, **kw):
        QtConfigurator.Forms.Configurator.__init__(self, *args, **kw)
        self.factory = factory.Factory(self.prepare_template("configuration"))

        # Get field data.
        # NOTE: This would be done for whole page updates in a Web application.

        self.populate_list(self.base_system, self.get_elements("base-system"))
        self.populate_list(self.keyboard, self.get_elements("keyboard"))
        self.populate_list(self.mouse, self.get_elements("mouse"))
        self.populate_list(self.screen, self.get_elements("screen"))

    # General functionality.

    def refresh(self, current_text=None):

        # Ensure consistency.
        # NOTE: This would be done for whole page updates in a Web application.
        # NOTE: This would also be done for page updates where the information
        # NOTE: involved was important.

        current_text = current_text or self.base_system.currentText()

        # Find the CPU socket and the interface of the current base system.
        cpu_socket = None
        interface = None
        for element in self.get_elements("base-system"):
            text = element.getAttribute("value")
            if text == current_text:
                cpu_socket = element.getAttribute("cpu-socket")
                interface = element.getAttribute("interface")

        # Find all valid CPUs.
        valid = []
        for element in self.get_elements("cpu"):
            if not element.hasAttribute("cpu-socket") or element.getAttribute("cpu-socket") == cpu_socket:
                valid.append(element)
        self.populate_list(self.cpu, valid)

        # Find all valid hard disks.
        valid = []
        for element in self.get_elements("hard-disk"):
            if not element.hasAttribute("interface") or element.getAttribute("interface") == interface:
                valid.append(element)
        for hard_disk_value in self.factory.find_widgets(self, "hard_disk_value"):
            self.populate_list(hard_disk_value, valid)

    # Slots.

    def baseSystemChanged(self, current_text):
        self.refresh(current_text)

    def addHardDisk(self):
        print self.sender()
        hard_disks = self.prepare_widget("configuration", "hard-disks")
        tab_pages = hard_disks.child("tab pages")
        tab = tab_pages.child("tab")
        self.hard_disks.addTab(tab, hard_disks.tabLabel(tab))
        self.factory.connect(tab, self)

        # Perform the consistency check.
        # NOTE: This is not as efficient as it could be since the general check
        # NOTE: refreshes all fields, not just newly added ones.
        self.refresh()

    def addMemoryUnit(self):
        memory_units = self.prepare_widget("configuration", "memory-units")
        tab_pages = memory_units.child("tab pages")
        tab = tab_pages.child("tab")
        self.memory_units.addTab(tab, memory_units.tabLabel(tab))
        self.factory.connect(tab, self)

        # Perform the consistency check.
        # NOTE: This is not as efficient as it could be since the general check
        # NOTE: refreshes all fields, not just newly added ones.
        self.refresh()

    def addStorageUnit(self):
        storage_units = self.prepare_widget("configuration", "storage-units")
        tab_pages = storage_units.child("tab pages")
        tab = tab_pages.child("tab")
        self.storage_units.addTab(tab, storage_units.tabLabel(tab))
        self.factory.connect(tab, self)

        # Perform the consistency check.
        # NOTE: This is not as efficient as it could be since the general check
        # NOTE: refreshes all fields, not just newly added ones.
        self.refresh()

    def removeHardDisk(self):
        page = self.hard_disks.currentPage()
        self.hard_disks.removePage(page)
        page.deleteLater()

    def removeMemoryUnit(self):
        page = self.memory_units.currentPage()
        self.memory_units.removePage(page)
        page.deleteLater()

    def removeStorageUnit(self):
        page = self.storage_units.currentPage()
        self.storage_units.removePage(page)
        page.deleteLater()

    def updateConfig(self):
        self.refresh()

    def exportConfig(self):
        print "configuration.exportConfig(): Not implemented yet"

# vim: tabstop=4 expandtab shiftwidth=4
